/*=============================================================================*/
/*
# Author: Max [ trlsmax@gmail.com ]
#
# Last modified:	2010-09-30 12:49
#
# Filename:		lcd.c
#
# Description: 1602LCD驱动
*/
/*=============================================================================*/
#include "lcd.h"

/* ========================================================================== */
/** 
 * @brief   lcd_delay 延时
 * 
 * @param   t
 */
/* ========================================================================== */
static void lcd_delay(uint8 t)
{
    uint8 i,j;
    for (i=t;i>0;i--)
        for (j=110;j>0;j--);
}

/* ========================================================================== */
/** 
 * @brief   lcd_write_cmd 写命令
 * 
 * @param   cmd 命令值
 */
/* ========================================================================== */
void lcd_write_cmd(uint8 cmd)
{
    //lcd_check_busy();
    lcd_rs = 0;
    lcd_rw = 0;
    DATA_PORT = cmd;
    lcd_delay(4);
    lcd_en = 1;
    lcd_delay(4);
    lcd_en = 0;
}
    
/* ========================================================================== */
/** 
 * @brief   lcd_write_dat 写数据
 * 
 * @param   dat 数据值
 */
/* ========================================================================== */
void lcd_write_dat(uint8 dat)
{
    //lcd_check_busy();
    lcd_rs = 1;
    lcd_rw = 0;
    DATA_PORT = dat;
    lcd_delay(4);
    lcd_en = 1;
    lcd_delay(4);
    lcd_en = 0;
}

/* ========================================================================== */
/** 
 * @brief   lcd_init LCD初始化
 */
/* ========================================================================== */
void lcd_init(void)
{
    lcd_rw = 0;
    lcd_en = 0;
    lcd_write_cmd(0x38);
    lcd_write_cmd(0x0C);
    lcd_write_cmd(0x06);
    lcd_write_cmd(0x01);
}

/* ========================================================================== */
/** 
 * @brief   lcd_set_xy 显示位置设定
 * 
 * @param   x 横坐标（0-15）
 * @param   y 纵坐标（0-1）
 */
/* ========================================================================== */
void lcd_set_xy(uint8 x, uint8 y)
{
    if (y == 0)
        lcd_write_cmd(0x80 + (x&0x0F));
    else if (y == 1)
        lcd_write_cmd(0xC0 + (x&0x0F));
}

/* ========================================================================== */
/** 
 * @brief   lcd_write_str 显示字符
 * 
 * @param   x 横坐标
 * @param   y 纵坐标
 * @param   str 需要显示的字符
 */
/* ========================================================================== */
void lcd_write_str(uint8 x, uint8 y, uint8 *str) reentrant
{
    uint8 i;

    lcd_set_xy(x,y);
    for (i=0;str[i] != '\0' && i+x < 16;i++)
        lcd_write_dat(str[i]);
}

/* ========================================================================== */
/** 
 * @brief   lcd_write_num 显示数字，由于电子快门只用到两位，所以只能支持到两位
 * 
 * @param   x 横坐标
 * @param   y 纵坐标
 * @param   num 要显示的数字
 * @param   n 位数（ONE/TWO）
 */
/* ========================================================================== */
void lcd_write_num(uint8 x, uint8 y, uint16 num,uint8 n) reentrant
{
    lcd_set_xy(x,y);
    if (n == ONE)
        lcd_write_dat(num%10+'0');
    else if (n == TWO)
    {
        lcd_write_dat(num/10%10+'0');
        lcd_write_dat(num%10+'0');
    }
    else if (n == THREE)
    {
        lcd_write_dat(num/100%10+'0');
        lcd_write_dat(num/10%10+'0');
        lcd_write_dat(num%10+'0');
    }
}
