/*=============================================================================*/
/*
# Author: Max [ trlsmax@gmail.com ]
#
# Last modified:	2010-09-30 12:49
#
# Filename:		main.c
#
# Description: 电子快门线
*/
/*=============================================================================*/
#include <main.h>

/* ============================================== */
/* 定义变量 */
/* ============================================== */
bit shuter_en;
bit delay_en;
uint8 code status[]="Status:";
uint8 code stage_array[][8] = {
    "Idle   ",
    "Setting",
    "Running",
    "Pause  ",
};
uint8 s=0;
uint8 key;
uint8 stage;
uint8 slot=0;
uint8 sel=SEL_SLOT;
int8 loop,hour,min,sec;
uint16 delay_cnt;
uint16 shuter_delay_cnt;
uint16 shuter_cnt;

struct _time_tag_ time[10] = {0};

/* ============================================== */
/* 函数声明 */
/* ============================================== */
void delay_t(uint16);
void sys_init(void);
uint8 get_key(void);
void display_slot(uint8 idx);

/* ============================================== */
/* 函数声明 */
/* ============================================== */
void main(void)
{
    uint8 tmp_key=NO_KEY;

    /* 系统初始化 */
    sys_init();
    /* LCD初始化 */
    lcd_init();
    delay_t(5);

    /* LCD界面初始化 */
    lcd_write_str(0,0,&status[0]);
    lcd_write_str(7,0,&stage_array[stage]);
    lcd_write_num(1,1,slot,TWO);
    display_slot(0);

    while(1)
    {
        delay_t(150);

        /* 按键扫描 */
        tmp_key = get_key();
        if (tmp_key == NO_KEY)
        {
            key = NO_KEY;
            continue;
        }
        delay_t(5);
        if (tmp_key != get_key())
            continue;

        /* 处理按键 */
        switch (tmp_key)
        {
            case KEY_0 :
                if (key == KEY_0)
                    break;

                /* 工作时暂停 */
                if (stage == S_RUNNING && shuter_en == TRUE)
                {
                    shuter_en = FALSE;
                    SHUTER_CLOSE();
                    stage = S_PAUSE;
                }
                /* 暂停后恢复 */
                else if (stage == S_PAUSE)
                {
                    /* 从下一轮开始曝光 */
                    if (--loop < 0)
                    {
                        time[slot].finish = TRUE;
                        /* 判断是否需要曝光 */
                        while (slot < NUM_OF_SLOT)
                        {
                            slot++;
                            if ((time[slot].hour + time[slot].min + time[slot].sec) != 0)
                            {
                                lcd_write_num(1,1,slot,TWO);
                                break;
                            }
                        }
                        /* 曝光结束 */
                        if (slot == NUM_OF_SLOT)
                        {
                            shuter_en = FALSE;
                            delay_en = FALSE;
                            stage = S_IDLE;
                            lcd_write_str(7,0,stage_array[S_IDLE]);
                        }
                        loop = time[slot].loop;
                    }
                    shuter_delay_cnt = 0;
                    delay_en = TRUE;
                    /* 重新装载时间数据 */
                    hour = time[slot].hour;
                    min = time[slot].min;
                    sec = time[slot].sec;

                    stage = S_RUNNING;
                }
                /* 状态切换 */
                else
                {
                    if (++stage == S_RUNNING)
                        stage = S_IDLE;

                }
                lcd_write_str(7,0,stage_array[stage]);

                /* 显示相应状态下的数据 */
                if (stage == S_SETTING)
                {
                    lcd_write_cmd(0x0F);
                    sel = SEL_SLOT;
                    lcd_write_num(1,1,slot,ONE);
                }
                else if (stage == S_PAUSE || stage == S_RUNNING)
                {
                    lcd_write_num(4,1,loop+1,THREE);
                    lcd_write_num(8,1,hour,TWO);
                    lcd_write_num(11,1,min,TWO);
                    lcd_write_num(14,1,sec,TWO);
                }
                else
                {
                    lcd_write_cmd(0xC);
                    display_slot(slot);
                }
                key = KEY_0;
                break;
            case KEY_1 :
                if (key == KEY_1)
                    break;
                
                /* 开始曝光 */
                if (stage == S_IDLE)
                {
                    stage = S_RUNNING;
                    slot = 0;
                    lcd_write_num(1,1,slot,TWO);
                    display_slot(slot);
                    shuter_cnt = 0;
                    loop = time[slot].loop;
                    hour = time[slot].hour;
                    min = time[slot].min;
                    sec = time[slot].sec;
                    lcd_write_str(7,0,stage_array[S_RUNNING]);
                    if ((hour + min + sec) != 0)
                        SHUTER_OPEN();
                    shuter_en = TRUE;
                }
                /* 在设置模式下选择要改变的数据 */
                else if (stage == S_SETTING)
                {
                    lcd_write_cmd(0x0F);
                    if (++sel == NO_SEL)
                        sel = SEL_SLOT;
                    switch (sel)
                    {
                        case SEL_SLOT :
                            lcd_set_xy(2,1);
                            break;
                        case SEL_LOOP100 :
                            lcd_set_xy(4,1);
                            break;
                        case SEL_LOOP :
                            lcd_set_xy(6,1);
                            break;
                        case SEL_HOUR :
                            lcd_set_xy(9,1);
                            break;
                        case SEL_MIN :
                            lcd_set_xy(12,1);
                            break;
                        case SEL_SEC :
                            lcd_set_xy(15,1);
                            break;
                        default :
                            break;
                    }
                }
                /* 停止曝光 */
                else if (stage == S_RUNNING)
                {
                    stage = S_IDLE;
                    shuter_en = FALSE;
                    SHUTER_CLOSE();
                    lcd_write_str(7,0,stage_array[S_IDLE]);
                }
                key = KEY_1;
                break;
            case KEY_2 :
                /* 修改数据 */
                if (stage == S_SETTING)
                {
                    lcd_write_cmd(0x0C);
                    switch (sel)
                    {
                        case SEL_SLOT :
                            if (slot < NUM_OF_SLOT)
                            {
                                lcd_write_num(1,1,++slot,TWO);
                                display_slot(slot);
                                lcd_write_cmd(0x0F);
                                lcd_set_xy(2,1);
                            }
                            break;
                        case SEL_LOOP100 :
                            if (time[slot].loop < 900)
                            {
                                time[slot].loop = time[slot].loop + 100;
                                lcd_write_num(4,1,time[slot].loop+1,THREE);
                                lcd_write_cmd(0x0F);
                                lcd_set_xy(4,1);
                            }
                            break;
                        case SEL_LOOP :
                            if (time[slot].loop < NUM_OF_LOOP)
                            {
                                lcd_write_num(4,1,++time[slot].loop+1,THREE);
                                lcd_write_cmd(0x0F);
                                lcd_set_xy(6,1);
                            }
                            break;
                        case SEL_HOUR :
                            if (time[slot].hour < 99)
                            {
                                lcd_write_num(8,1,++time[slot].hour,TWO);
                                lcd_write_cmd(0x0F);
                                lcd_set_xy(9,1);
                            }
                            break;
                        case SEL_MIN :
                            if (time[slot].min < 59)
                            {
                                lcd_write_num(11,1,++time[slot].min,TWO);
                                lcd_write_cmd(0x0F);
                                lcd_set_xy(12,1);
                            }
                            break;
                        case SEL_SEC :
                            if (time[slot].sec < 59)
                            {
                                lcd_write_num(14,1,++time[slot].sec,TWO);
                                lcd_write_cmd(0x0F);
                                lcd_set_xy(15,1);
                            }
                            break;
                        default :
                            break;
                    }
                }
                key = KEY_2;
                break;
            case KEY_3 :
                /* 修改数据 */
                if (stage == S_SETTING)
                {
                    lcd_write_cmd(0x0C);
                    switch (sel)
                    {
                        case SEL_SLOT :
                            if (slot > 0)
                            {
                                lcd_write_num(1,1,--slot,TWO);
                                display_slot(slot);
                                lcd_write_cmd(0x0F);
                                lcd_set_xy(2,1);
                            }
                            break;
                        case SEL_LOOP100 :
                            if (time[slot].loop > 100)
                            {
                                time[slot].loop -= 100;
                                lcd_write_num(4,1,time[slot].loop+1,THREE);
                                lcd_write_cmd(0x0F);
                                lcd_set_xy(4,1);
                            }
                            break;
                        case SEL_LOOP :
                            if (time[slot].loop > 0)
                            {
                                lcd_write_num(4,1,--time[slot].loop+1,THREE);
                                lcd_write_cmd(0x0F);
                                lcd_set_xy(6,1);
                            }
                            break;
                        case SEL_HOUR :
                            if (time[slot].hour > 0)
                            {
                                lcd_write_num(8,1,--time[slot].hour,TWO);
                                lcd_write_cmd(0x0F);
                                lcd_set_xy(9,1);
                            }
                            break;
                        case SEL_MIN :
                            if (time[slot].min > 0)
                            {
                                lcd_write_num(11,1,--time[slot].min,TWO);
                                lcd_write_cmd(0x0F);
                                lcd_set_xy(12,1);
                            }
                            break;
                        case SEL_SEC :
                            if (time[slot].sec > 0)
                            {
                                lcd_write_num(14,1,--time[slot].sec,TWO);
                                lcd_write_cmd(0x0F);
                                lcd_set_xy(15,1);
                            }
                            break;
                        default :
                            break;
                    }
                }
                key = KEY_3;
                break;
            case KEY_4 :
                /* 背光控制 */
                if (key == KEY_4)
                    break;
                led = !led;
                key = KEY_4;
                break;
            default :
                break;
        }
    }
}

/* ========================================================================== */
/** 
 * @brief   sys_init 系统初始化
 */
/* ========================================================================== */
void sys_init(void)
{
    /* 定时器0设置 */
	TMOD = 0x01;
	TH0 = 0xFC;
	TL0 = 0x5C;
    ET0 = 1;
    TR0 = 1;

    /* 开中断 */
    EA = 1;

    /* 开机马上打开LCD背光 */
    led = 0;
}    

/* ========================================================================== */
/** 
 * @brief   timer0_it 定时器中断处理
 */
/* ========================================================================== */
void timer0_it(void) interrupt 1
{
    /* 重新装载 */
	TH0 = 0xFC;
	TL0 = 0x5C;

    STOP_TIMER0();
    test = !test;
    delay_cnt--;

    if (shuter_en)
    {
        /* 曝光倒计时 */
       if (shuter_cnt++ > 1000)
       {
           shuter_cnt = 0;
           sec--;
           if (sec < 0 && (min != 0 || hour != 0))
           {
               sec = 59;
               min--;
               if (min < 0 && hour != 0)
               {
                   min = 59;
                   hour--;
                   lcd_write_num(8,1,hour,TWO);
               }
               lcd_write_num(11,1,min,TWO);
           }
           /* 一次曝光结束 */
           else if (sec < 0)
           {
               SHUTER_CLOSE();
               shuter_en = FALSE;
               /* 进入下一轮曝光 */
               if (--loop < 0)
               {
                   time[slot].finish = TRUE;
                   /* 检查是否需要曝光 */
                   while (slot < NUM_OF_SLOT)
                   {
                       slot++;
                       if ((time[slot].hour + time[slot].min + time[slot].sec) != 0)
                       {
                           lcd_write_num(1,1,slot,TWO);
                           break;
                       }
                   }
                   /* 完成曝光 */
                   if (slot == NUM_OF_SLOT)
                   {
                       shuter_en = FALSE;
                       delay_en = FALSE;
                       stage = S_IDLE;
                       slot = 0;
                       lcd_write_str(7,0,stage_array[S_IDLE]);
                   }
                   loop = time[slot].loop;
               }
               shuter_delay_cnt = 0;
               if (stage == S_RUNNING)
                   delay_en = TRUE;
               lcd_write_num(4,1,loop+1,THREE);
               /* 装载新一轮的曝光时间 */
               hour = time[slot].hour;
               min = time[slot].min;
               sec = time[slot].sec;
           }
           lcd_write_num(14,1,sec,TWO);
       }
    }

    /* 曝光间延时500ms */
    if (delay_en)
    {
        if (++shuter_delay_cnt > 500)
        {
            SHUTER_OPEN();
            shuter_en = TRUE;
            delay_en = FALSE;
        }
    }
    START_TIMER0();
}

/* ========================================================================== */
/** 
 * @brief   delay_t 延时函数
 * 
 * @param   t 需要延时的ms数
 */
/* ========================================================================== */
void delay_t(uint16 t)
{
    STOP_TIMER0();
    delay_cnt = t;
    START_TIMER0();
    while(delay_cnt > 1);
}

/* ========================================================================== */
/** 
 * @brief   get_key 获取按键值
 * 
 * @return  返回获取的按键值
 */
/* ========================================================================== */
uint8 get_key(void)
{
    if (!key0)
        return KEY_0;
    else if (!key1)
        return KEY_1;
    else if (!key2)
        return KEY_2;
    else if (!key3)
        return KEY_3;
    else if (!key4)
        return KEY_4;
    else
        return NO_KEY;
}

/* ========================================================================== */
/** 
 * @brief   display_slot 显示时间
 * 
 * @param   idx 需要显示的组
 */
/* ========================================================================== */
void display_slot(uint8 idx)
{
    lcd_write_num(4,1,time[idx].loop+1,THREE);
    lcd_write_num(8,1,time[idx].hour,TWO);
    lcd_write_str(10,1,":");
    lcd_write_num(11,1,time[idx].min,TWO);
    lcd_write_str(13,1,":");
    lcd_write_num(14,1,time[idx].sec,TWO);
}


